==> bcoin-native-0.0.1.tgz: package/src/siphash.cc <==
#include "siphash.h"
#include "common.h"

#define ROTL(x, b) (uint64_t)(((x) << (b)) | ((x) >> (64 - (b))))

#define SIPROUND do { \
  v0 += v1; v1 = ROTL(v1, 13); v1 ^= v0; \
  v0 = ROTL(v0, 32); \
  v2 += v3; v3 = ROTL(v3, 16); v3 ^= v2; \
  v0 += v3; v3 = ROTL(v3, 21); v3 ^= v0; \
  v2 += v1; v1 = ROTL(v1, 17); v1 ^= v2; \
  v2 = ROTL(v2, 32); \
} while (0)

void
bcn_siphash24(
  const uint8_t *data,
  size_t len,
  const uint8_t *key,
  uint8_t *out,
  uint8_t shift
) {
  uint64_t k0 = READU64(key + 0);
  uint64_t k1 = READU64(key + 8);
  uint32_t blocks = len / 8;
  uint64_t v0 = 0x736f6d6570736575ULL ^ k0;
  uint64_t v1 = 0x646f72616e646f6dULL ^ k1;
  uint64_t v2 = 0x6c7967656e657261ULL ^ k0;
  uint64_t v3 = 0x7465646279746573ULL ^ k1;
  uint64_t f0 = ((uint64_t)blocks << shift);
  const uint64_t f1 = 0xff;
  uint64_t d;

  for (uint32_t i = 0; i < blocks; i++) {
    d = READU64(data);
    data += 8;
    v3 ^= d;
    SIPROUND;
    SIPROUND;
    v0 ^= d;
  }

  switch (len & 7) {
    case 7:
      f0 |= ((uint64_t)data[6]) << 48;
    case 6:
      f0 |= ((uint64_t)data[5]) << 40;
    case 5:
      f0 |= ((uint64_t)data[4]) << 32;
    case 4:
      f0 |= ((uint64_t)data[3]) << 24;
    case 3:
      f0 |= ((uint64_t)data[2]) << 16;
    case 2:
      f0 |= ((uint64_t)data[1]) << 8;
    case 1:
      f0 |= ((uint64_t)data[0]);
      break;
    case 0:
      break;
  }

  v3 ^= f0;
  SIPROUND;
  SIPROUND;
  v0 ^= f0;
  v2 ^= f1;
  SIPROUND;
  SIPROUND;
  SIPROUND;
  SIPROUND;
  v0 ^= v1;
  v0 ^= v2;
  v0 ^= v3;

  WRITEU64(out, v0);
}

void
bcn_siphash(const uint8_t *data, size_t len, const uint8_t *key, uint8_t *out) {
  return bcn_siphash24(data, len, key, out, 56);
}

void
bcn_siphash256(const uint8_t *data, size_t len, const uint8_t *key, uint8_t *out) {
  return bcn_siphash24(data, len, key, out, 59);
}

==> bcoin-native-0.0.20.tgz: package/src/siphash.cc <==
#include "siphash.h"
#include "common.h"

#define ROTL(x, b) (uint64_t)(((x) << (b)) | ((x) >> (64 - (b))))

#define SIPROUND do { \
  v0 += v1; v1 = ROTL(v1, 13); v1 ^= v0; \
  v0 = ROTL(v0, 32); \
  v2 += v3; v3 = ROTL(v3, 16); v3 ^= v2; \
  v0 += v3; v3 = ROTL(v3, 21); v3 ^= v0; \
  v2 += v1; v1 = ROTL(v1, 17); v1 ^= v2; \
  v2 = ROTL(v2, 32); \
} while (0)

uint64_t
bcn_siphash24(
  const uint8_t *data,
  size_t len,
  const uint8_t *key,
  uint8_t shift
) {
  uint64_t k0 = READU64(key + 0);
  uint64_t k1 = READU64(key + 8);
  uint32_t blocks = len / 8;
  uint64_t v0 = 0x736f6d6570736575ULL ^ k0;
  uint64_t v1 = 0x646f72616e646f6dULL ^ k1;
  uint64_t v2 = 0x6c7967656e657261ULL ^ k0;
  uint64_t v3 = 0x7465646279746573ULL ^ k1;
  uint64_t f0 = ((uint64_t)blocks << shift);
  const uint64_t f1 = 0xff;
  uint64_t d;

  for (uint32_t i = 0; i < blocks; i++) {
    d = READU64(data);
    data += 8;
    v3 ^= d;
    SIPROUND;
    SIPROUND;
    v0 ^= d;
  }

  switch (len & 7) {
    case 7:
      f0 |= ((uint64_t)data[6]) << 48;
    case 6:
      f0 |= ((uint64_t)data[5]) << 40;
    case 5:
      f0 |= ((uint64_t)data[4]) << 32;
    case 4:
      f0 |= ((uint64_t)data[3]) << 24;
    case 3:
      f0 |= ((uint64_t)data[2]) << 16;
    case 2:
      f0 |= ((uint64_t)data[1]) << 8;
    case 1:
      f0 |= ((uint64_t)data[0]);
      break;
    case 0:
      break;
  }

  v3 ^= f0;
  SIPROUND;
  SIPROUND;
  v0 ^= f0;
  v2 ^= f1;
  SIPROUND;
  SIPROUND;
  SIPROUND;
  SIPROUND;
  v0 ^= v1;
  v0 ^= v2;
  v0 ^= v3;

  return v0;
}

uint64_t
bcn_siphash(const uint8_t *data, size_t len, const uint8_t *key) {
  return bcn_siphash24(data, len, key, 56);
}

uint64_t
bcn_siphash256(const uint8_t *data, size_t len, const uint8_t *key) {
  return bcn_siphash24(data, len, key, 59);
}
